# Java Ecosystem Capabilities Gradle plugin

[![Build Status](https://img.shields.io/endpoint.svg?url=https%3A%2F%2Factions-badge.atrox.dev%2Fgradlex-org%2Fjava-ecosystem-capabilities%2Fbadge%3Fref%3Dmain&style=flat)](https://actions-badge.atrox.dev/gradlex-org/java-ecosystem-capabilities/goto?ref=main)
[![Gradle Plugin Portal](https://img.shields.io/maven-metadata/v?label=Plugin%20Portal&metadataUrl=https%3A%2F%2Fplugins.gradle.org%2Fm2%2Forg%2Fgradlex%2Fjava-ecosystem-capabilities%2Forg.gradlex.java-ecosystem-capabilities.gradle.plugin%2Fmaven-metadata.xml)](https://plugins.gradle.org/plugin/org.gradlex.java-ecosystem-capabilities)

This plugin adds [Capabilities](https://docs.gradle.org/current/userguide/component_capabilities.html#capabilities_as_first_level_concept) 
to the metadata of well-known components hosted on Maven Central that are used in many Java projects.

## What is a 'Capability' in Gradle and why should I care?

In [the video below](https://www.youtube.com/watch?v=5g20kbbqBFk&list=PLWQK2ZdV4Yl2k2OmC_gsjDpdIBTN0qqkE), 
I explain the concept of Capability Conflicts and why they can help you to avoid "dependency hell" in your project.
With this plugin, you enable Gradle to detect and automatically resolved typical capability conflicts in the Java Ecosystem.

[<img src="https://onepiecesoftware.github.io/img/videos/11.png" width="320">](https://www.youtube.com/watch?v=5g20kbbqBFk&list=PLWQK2ZdV4Yl2k2OmC_gsjDpdIBTN0qqkE)

## How to use the plugin?

Apply the plugin to all (sub)projects of your build so that the capability-adding rules are active everywhere.
There is nothing else you need to do. The rules will now be considered when dependencies are resolved.
For general information about how to structure Gradle builds, and apply community plugins like this one to all subprojects,
you can check out my [Understanding Gradle video series](https://www.youtube.com/watch?v=N95YI-szd78&list=PLWQK2ZdV4Yl2k2OmC_gsjDpdIBTN0qqkE).

### Plugin dependency

Add this to the build file of your convention plugin's build
(e.g. `build-logic/build.gradle(.kts)` or `buildSrc/build.gradle(.kts)`).

```
dependencies {
    implementation("org.gradlex:java-ecosystem-capabilities:1.0")
}
```

### Apply the plugin

In your convention plugin, apply the plugin.

```
plugins {
    ...
    id("org.gradlex.java-ecosystem-capabilities")
}
```

You can apply the plugin as a _project plugin_ (`build.gradle(.kts)`) or a _setting plugin_ (`settings.gradle(.kts)`).
If you don't know what the difference is, using it as _project plugin_ (most Gradle plugins are  _project plugins_) is the preferred way.

The following things are to consider:
- If you use it as a _project plugin_ make sure to apply it to **all** (sub)projects using a convention plugin
- If you use it as a _settings plugin_ you may directly apply it in your `settings.gradle(.kts)`
- If you use it as a _settings plugin_, there are no [default resolution strategies](#deactivating-the-default-resolution-strategy-for-capabilities) registered
- If you write additional component metadata rules, or use other plugins that add more rules, make sure to consistently do either everything in _projects_ or in _settings_ as Gradle has trouble combining both approaches.

### Supported Gradle versions

- Minimal Gradle version when used as _project plugin_ is `6.0`
- Minimal Gradle version when used as _settings plugin_ is `6.8`

Note: There are rules in this plugin (in particular for _Guava_) which work _better_ with Gradle `7+`.
This is because Gradle 7 added support for the `org.gradle.jvm.environment` Attribute that allows Gradle to better distinguish between _standard-jvm_ and _android_ projects.

### Alternative: Copy rules into your build logic

Instead of applying this plugin, you may also copy selected rules to your own _build logic_ and register them in your convention plugin(s) directly.
In [the list below](#which-components-does-this-plugin-affect), all rule implementations are linked.
Here they are implemented in Java, but converting them to Kotlin or Groovy should be straightforward if you prefer.

### Deactivating the default resolution strategy for capabilities

The plugin adds a _default resolution strategy_ for each capability (if the plugin is used as _project plugin_).
Which in most cases means that Gradle will automatically pick the highest version of all components in conflict.
This is to cover the cases where users _just want things to work somehow_.
But it might not always be the right solution.

If you prefer to get all the conflicts reported and then add the conflict resolution explicitly (see next section),
you can deactivate the default resolution for one or all rules through the `javaEcosystemCapabilities` extension.

```
// Deactivate default resolution strategy for all rules
javaEcosystemCapabilities {
    deactivatedResolutionStrategies.addAll(allCapabilities)
}

// Deactivate default resolution strategy for selected rules
javaEcosystemCapabilities {
    deactivatedResolutionStrategies.addAll(CGlibRule.CAPABILITY)
    deactivatedResolutionStrategies.addAll(JavaxMailApiRule.CAPABILITY)
}
```

### I use the plugin and now there is a conflict - what now?

Unless deactivated (see above), the plugin configures Gradle to resolve conflicts by selecting the highest version.
If Gradle does not know how to resolve a conflict, you get a conflict error.

If you get an error like this:

```
> Module 'com.sun.mail:jakarta.mail' has been rejected:
     Cannot select module with conflict on capability 'javax.mail:mail:2.0.1' also provided by [com.sun.mail:mailapi:2.0.1(compile)]
```

It means that you need to make a decision for the given capability - in this case `javax.mail:mail` - by selecting one of the modules that both provide the capability.
In this case, you can decide between `com.sun.mail:jakarta.mail` (see first line of message) and `com.sun.mail:mailapi` (see end of second line).

A decision is made by defining a resolution strategy for the capability.
This is best done in the place where you applied this plugin (e.g. one of your convention plugins):

```
configurations.all {
  resolutionStrategy.capabilitiesResolution {
    withCapability("javax.mail:mail") {        // Capability for which to make the decision
      select("com.sun.mail:jakarta.mail:0")    // The component to select
    }
  }
}
```

### One of the rules added by the plugin has an undesired effect - what now?

The goal of this plugin is to enrich the metadata of widely used components from Maven Central to allow Gradle to detect conflicts.
The rules implemented in this plugin extend existing metadata with the mindset that the metadata should look like that in the first place.
It just doesn't for technical limitations during the development of the component.
In most cases, because the component is published with Maven and only published POM metadata which cannot express capability information.

With that in mind, the rules _should_ be usable as they are for almost all Gradle builds.
If you encounter a problem with a rule in your build:

1. Maybe there is a mistake/bug in this plugin. Please [open an issue](https://github.com/gradlex-org/java-ecosystem-capabilities/blob/main/issues/new) to discuss it.
2. You might have a very special setup, where one of the rules causes trouble only in that setup.

In the second case, you cannot deactivate one of the rules in the plugin.
But you can treat the modified metadata as if it was the original metadata and add another rule on top to modify it further or to revert the effect of the rule in this plugin.
For example:

```
dependencies {
    components.withModule(CGlibRule.MODULES[0]) {
        // Ad-hoc rule to revert the effect of 'CGlibRule'
        allVariants {
            withCapabilities {
                removeCapability(CGlibRule.CAPABILITY_GROUP, CGlibRule.CAPABILITY_NAME)
            }
        }
    }
}
```

Such additional rules are best added in the place where you applied this plugin (e.g. one of your convention plugins).
The snippet above shows how to add a rule without putting it into a separate class.
You can put it into a class (written in Java, Kotlin or Groovy) and use the `@CacheableRule` annotation for better performance.
That's how the rules in this plugin are implemented.
Consult the Gradle documentation on [Component Metadata Rules](https://docs.gradle.org/current/userguide/component_metadata_rules.html) for more details.

## What is the concrete effect of the plugin?

The plugin makes sure that during dependency resolution, you do not end up with two components that 'do the same thing' in the dependency resolution result.
That is, you won't have two Jars with different names (e.g. `jakarta.xml.bind-api-3.0.1.jar` and `jaxb-api-2.3.1.jar`) but same/similar classes on the classpath.
In this example, Gradle will use `jakarta.xml.bind-api` in place of `jaxb-api` in all places.
You can see all effects in [this build scan](https://scans.gradle.com/s/d45kd3g3uztlu/dependencies?toggled=W1swXSxbMCwxXV0)
from this [artificial sample project](samples/sample-all/build.gradle.kts) that includes dependencies to all components covered by rules in this plugin.


[<img src="samples/sample-all/build-scan.png">](https://scans.gradle.com/s/d45kd3g3uztlu/dependencies?toggled=W1swXSxbMCwxXV0)

## Which Components does this plugin affect?

The following list shows all Capabilities and the Components they are added to.
Each Capability's GA coordinates correspond to the GA coordinates of the Component that first introduced the Capability.

<!-- START_GENERATED -->
* [asm:asm](https://mvnrepository.com/artifact/asm/asm) ([AsmRule](src/main/java/org.gradlex/javaecosystem/capabilities/rules/AsmRule.java))
  * [org.ow2.asm:asm](https://mvnrepository.com/artifact/org.ow2.asm/asm)
* [c3p0:c3p0](https://mvnrepository.com/artifact/c3p0/c3p0) ([C3p0Rule](src/main/java/org.gradlex/javaecosystem/capabilities/rules/C3p0Rule.java))
  * [com.mchange:c3p0](https://mvnrepository.com/artifact/com.mchange/c3p0)
* [cglib:cglib](https://mvnrepository.com/artifact/cglib/cglib) ([CGlibRule](src/main/java/org.gradlex/javaecosystem/capabilities/rules/CGlibRule.java))
  * [cglib:cglib-nodep](https://mvnrepository.com/artifact/cglib/cglib-nodep)
* [com.google.collections:google-collections](https://mvnrepository.com/artifact/com.google.collections/google-collections) ([GoogleCollectionsRule](src/main/java/org.gradlex/javaecosystem/capabilities/rules/GoogleCollectionsRule.java))
  * [com.google.guava:guava](https://mvnrepository.com/artifact/com.google.guava/guava)
* [com.google.guava:guava](https://mvnrepository.com/artifact/com.google.guava/guava) ([GuavaRule](src/main/java/org.gradlex/javaecosystem/capabilities/rules/GuavaRule.java))
  * [com.google.guava:guava-jdk5](https://mvnrepository.com/artifact/com.google.guava/guava-jdk5)
* [com.google.guava:listenablefuture](https://mvnrepository.com/artifact/com.google.guava/listenablefuture) ([GuavaListenableFutureRule](src/main/java/org.gradlex/javaecosystem/capabilities/rules/GuavaListenableFutureRule.java))
  * [com.google.guava:guava](https://mvnrepository.com/artifact/com.google.guava/guava)
* [com.vividsolutions:jts-core](https://mvnrepository.com/artifact/com.vividsolutions/jts-core) ([JtsCoreRule](src/main/java/org.gradlex/javaecosystem/capabilities/rules/JtsCoreRule.java))
  * [org.locationtech.jts:jts-core](https://mvnrepository.com/artifact/org.locationtech.jts/jts-core)
* [com.vividsolutions:jts](https://mvnrepository.com/artifact/com.vividsolutions/jts) ([JtsRule](src/main/java/org.gradlex/javaecosystem/capabilities/rules/JtsRule.java))
  * [org.locationtech.jts:jts-core](https://mvnrepository.com/artifact/org.locationtech.jts/jts-core)
* [commons-io:commons-io](https://mvnrepository.com/artifact/commons-io/commons-io) ([CommonsIoRule](src/main/java/org.gradlex/javaecosystem/capabilities/rules/CommonsIoRule.java))
  * [org.apache.commons:commons-io](https://mvnrepository.com/artifact/org.apache.commons/commons-io)
* [dom4j:dom4j](https://mvnrepository.com/artifact/dom4j/dom4j) ([Dom4jRule](src/main/java/org.gradlex/javaecosystem/capabilities/rules/Dom4jRule.java))
  * [org.dom4j:dom4j](https://mvnrepository.com/artifact/org.dom4j/dom4j)
* [jakarta.activation:jakarta.activation-api](https://mvnrepository.com/artifact/jakarta.activation/jakarta.activation-api) ([JakartaActivationApiRule](src/main/java/org.gradlex/javaecosystem/capabilities/rules/JakartaActivationApiRule.java))
  * [com.sun.activation:jakarta.activation](https://mvnrepository.com/artifact/com.sun.activation/jakarta.activation)
* [jakarta.annotation:jakarta.annotation-api](https://mvnrepository.com/artifact/jakarta.annotation/jakarta.annotation-api) ([JakartaAnnotationApiRule](src/main/java/org.gradlex/javaecosystem/capabilities/rules/JakartaAnnotationApiRule.java))
  * [org.apache.tomcat:tomcat-annotations-api](https://mvnrepository.com/artifact/org.apache.tomcat/tomcat-annotations-api)
* [jakarta.mail:jakarta.mail-api](https://mvnrepository.com/artifact/jakarta.mail/jakarta.mail-api) ([JakartaMailApiRule](src/main/java/org.gradlex/javaecosystem/capabilities/rules/JakartaMailApiRule.java))
  * [com.sun.mail:mailapi](https://mvnrepository.com/artifact/com.sun.mail/mailapi)
  * [com.sun.mail:jakarta.mail](https://mvnrepository.com/artifact/com.sun.mail/jakarta.mail)
  * [org.eclipse.angus:jakarta.mail](https://mvnrepository.com/artifact/org.eclipse.angus/jakarta.mail)
* [jakarta.servlet:jakarta.servlet-api](https://mvnrepository.com/artifact/jakarta.servlet/jakarta.servlet-api) ([JakartaServletApiRule](src/main/java/org.gradlex/javaecosystem/capabilities/rules/JakartaServletApiRule.java))
  * [org.apache.tomcat:tomcat-servlet-api](https://mvnrepository.com/artifact/org.apache.tomcat/tomcat-servlet-api)
* [jakarta.ws.rs:jakarta.ws.rs-api](https://mvnrepository.com/artifact/jakarta.ws.rs/jakarta.ws.rs-api) ([JakartaWsRsApiRule](src/main/java/org.gradlex/javaecosystem/capabilities/rules/JakartaWsRsApiRule.java))
  * [org.jboss.spec.javax.ws.rs:jboss-jaxrs-api_3.0_spec](https://mvnrepository.com/artifact/org.jboss.spec.javax.ws.rs/jboss-jaxrs-api_3.0_spec)
* [javassist:javassist](https://mvnrepository.com/artifact/javassist/javassist) ([JavaAssistRule](src/main/java/org.gradlex/javaecosystem/capabilities/rules/JavaAssistRule.java))
  * [org.javassist:javassist](https://mvnrepository.com/artifact/org.javassist/javassist)
  * [jboss:javassist](https://mvnrepository.com/artifact/jboss/javassist)
* [javax.activation:activation](https://mvnrepository.com/artifact/javax.activation/activation) ([JavaxActivationApiRule](src/main/java/org.gradlex/javaecosystem/capabilities/rules/JavaxActivationApiRule.java))
  * [jakarta.activation:jakarta.activation-api](https://mvnrepository.com/artifact/jakarta.activation/jakarta.activation-api)
  * [com.sun.activation:javax.activation](https://mvnrepository.com/artifact/com.sun.activation/javax.activation)
  * [com.sun.activation:jakarta.activation](https://mvnrepository.com/artifact/com.sun.activation/jakarta.activation)
* [javax.annotation:jsr250-api](https://mvnrepository.com/artifact/javax.annotation/jsr250-api) ([JavaxAnnotationApiRule](src/main/java/org.gradlex/javaecosystem/capabilities/rules/JavaxAnnotationApiRule.java))
  * [jakarta.annotation:jakarta.annotation-api](https://mvnrepository.com/artifact/jakarta.annotation/jakarta.annotation-api)
  * [javax.annotation:javax.annotation-api](https://mvnrepository.com/artifact/javax.annotation/javax.annotation-api)
  * [org.apache.tomcat:tomcat-annotations-api](https://mvnrepository.com/artifact/org.apache.tomcat/tomcat-annotations-api)
* [javax.ejb:ejb](https://mvnrepository.com/artifact/javax.ejb/ejb) ([JavaxEjbApiRule](src/main/java/org.gradlex/javaecosystem/capabilities/rules/JavaxEjbApiRule.java))
  * [jakarta.ejb:jakarta.ejb-api](https://mvnrepository.com/artifact/jakarta.ejb/jakarta.ejb-api)
  * [javax.ejb:javax.ejb-api](https://mvnrepository.com/artifact/javax.ejb/javax.ejb-api)
  * [javax.ejb:ejb-api](https://mvnrepository.com/artifact/javax.ejb/ejb-api)
* [javax.el:el-api](https://mvnrepository.com/artifact/javax.el/el-api) ([JavaxElApiRule](src/main/java/org.gradlex/javaecosystem/capabilities/rules/JavaxElApiRule.java))
  * [jakarta.el:jakarta.el-api](https://mvnrepository.com/artifact/jakarta.el/jakarta.el-api)
  * [javax.el:javax.el-api](https://mvnrepository.com/artifact/javax.el/javax.el-api)
* [javax.inject:javax.inject](https://mvnrepository.com/artifact/javax.inject/javax.inject) ([JavaxInjectApiRule](src/main/java/org.gradlex/javaecosystem/capabilities/rules/JavaxInjectApiRule.java))
  * [jakarta.inject:jakarta.inject-api](https://mvnrepository.com/artifact/jakarta.inject/jakarta.inject-api)
* [javax.jws:javax.jws-api](https://mvnrepository.com/artifact/javax.jws/javax.jws-api) ([JavaxJwsApisRule](src/main/java/org.gradlex/javaecosystem/capabilities/rules/JavaxJwsApisRule.java))
  * [jakarta.jws:jakarta.jws-api](https://mvnrepository.com/artifact/jakarta.jws/jakarta.jws-api)
* [javax.mail:mail](https://mvnrepository.com/artifact/javax.mail/mail) ([JavaxMailApiRule](src/main/java/org.gradlex/javaecosystem/capabilities/rules/JavaxMailApiRule.java))
  * [com.sun.mail:mailapi](https://mvnrepository.com/artifact/com.sun.mail/mailapi)
  * [jakarta.mail:jakarta.mail-api](https://mvnrepository.com/artifact/jakarta.mail/jakarta.mail-api)
  * [com.sun.mail:javax.mail](https://mvnrepository.com/artifact/com.sun.mail/javax.mail)
  * [com.sun.mail:jakarta.mail](https://mvnrepository.com/artifact/com.sun.mail/jakarta.mail)
* [javax.persistence:javax.persistence-api](https://mvnrepository.com/artifact/javax.persistence/javax.persistence-api) ([JavaxPersistenceApiRule](src/main/java/org.gradlex/javaecosystem/capabilities/rules/JavaxPersistenceApiRule.java))
  * [jakarta.persistence:jakarta.persistence-api](https://mvnrepository.com/artifact/jakarta.persistence/jakarta.persistence-api)
  * [org.hibernate.javax.persistence:hibernate-jpa-2.2-api](https://mvnrepository.com/artifact/org.hibernate.javax.persistence/hibernate-jpa-2.2-api)
  * [org.hibernate.javax.persistence:hibernate-jpa-2.1-api](https://mvnrepository.com/artifact/org.hibernate.javax.persistence/hibernate-jpa-2.1-api)
  * [org.hibernate.javax.persistence:hibernate-jpa-2.0-api](https://mvnrepository.com/artifact/org.hibernate.javax.persistence/hibernate-jpa-2.0-api)
* [javax.servlet:servlet-api](https://mvnrepository.com/artifact/javax.servlet/servlet-api) ([JavaxServletApiRule](src/main/java/org.gradlex/javaecosystem/capabilities/rules/JavaxServletApiRule.java))
  * [javax.servlet:javax.servlet-api](https://mvnrepository.com/artifact/javax.servlet/javax.servlet-api)
  * [jakarta.servlet:jakarta.servlet-api](https://mvnrepository.com/artifact/jakarta.servlet/jakarta.servlet-api)
  * [org.apache.tomcat:servlet-api](https://mvnrepository.com/artifact/org.apache.tomcat/servlet-api)
  * [org.apache.tomcat:tomcat-servlet-api](https://mvnrepository.com/artifact/org.apache.tomcat/tomcat-servlet-api)
* [javax.validation:validation-api](https://mvnrepository.com/artifact/javax.validation/validation-api) ([JavaxValidationApiRule](src/main/java/org.gradlex/javaecosystem/capabilities/rules/JavaxValidationApiRule.java))
  * [jakarta.validation:jakarta.validation-api](https://mvnrepository.com/artifact/jakarta.validation/jakarta.validation-api)
* [javax.ws.rs:jsr311-api](https://mvnrepository.com/artifact/javax.ws.rs/jsr311-api) ([JavaxWsRsApiRule](src/main/java/org.gradlex/javaecosystem/capabilities/rules/JavaxWsRsApiRule.java))
  * [org.jboss.spec.javax.ws.rs:jboss-jaxrs-api_2.1_spec](https://mvnrepository.com/artifact/org.jboss.spec.javax.ws.rs/jboss-jaxrs-api_2.1_spec)
  * [org.jboss.spec.javax.ws.rs:jboss-jaxrs-api_2.0_spec](https://mvnrepository.com/artifact/org.jboss.spec.javax.ws.rs/jboss-jaxrs-api_2.0_spec)
  * [org.jboss.spec.javax.ws.rs:jboss-jaxrs-api_1.1_spec](https://mvnrepository.com/artifact/org.jboss.spec.javax.ws.rs/jboss-jaxrs-api_1.1_spec)
  * [org.jboss.resteasy:jaxrs-api](https://mvnrepository.com/artifact/org.jboss.resteasy/jaxrs-api)
  * [jakarta.ws.rs:jakarta.ws.rs-api](https://mvnrepository.com/artifact/jakarta.ws.rs/jakarta.ws.rs-api)
  * [javax.ws.rs:javax.ws.rs-api](https://mvnrepository.com/artifact/javax.ws.rs/javax.ws.rs-api)
* [javax.xml.bind:jaxb-api](https://mvnrepository.com/artifact/javax.xml.bind/jaxb-api) ([JavaxXmlBindApiRule](src/main/java/org.gradlex/javaecosystem/capabilities/rules/JavaxXmlBindApiRule.java))
  * [jakarta.xml.bind:jakarta.xml.bind-api](https://mvnrepository.com/artifact/jakarta.xml.bind/jakarta.xml.bind-api)
* [javax.xml.soap:javax.xml.soap-api](https://mvnrepository.com/artifact/javax.xml.soap/javax.xml.soap-api) ([JavaxSoapApiRule](src/main/java/org.gradlex/javaecosystem/capabilities/rules/JavaxSoapApiRule.java))
  * [jakarta.xml.soap:jakarta.xml.soap-api](https://mvnrepository.com/artifact/jakarta.xml.soap/jakarta.xml.soap-api)
* [javax.xml.ws:jaxws-api](https://mvnrepository.com/artifact/javax.xml.ws/jaxws-api) ([JavaxXmlWsApiRule](src/main/java/org.gradlex/javaecosystem/capabilities/rules/JavaxXmlWsApiRule.java))
  * [jakarta.xml.ws:jakarta.xml.ws-api](https://mvnrepository.com/artifact/jakarta.xml.ws/jakarta.xml.ws-api)
* [junit:junit](https://mvnrepository.com/artifact/junit/junit) ([JunitRule](src/main/java/org.gradlex/javaecosystem/capabilities/rules/JunitRule.java))
  * [junit:junit-dep](https://mvnrepository.com/artifact/junit/junit-dep)
* [org.hamcrest:hamcrest-core](https://mvnrepository.com/artifact/org.hamcrest/hamcrest-core) ([HamcrestCoreRule](src/main/java/org.gradlex/javaecosystem/capabilities/rules/HamcrestCoreRule.java))
  * [org.hamcrest:hamcrest](https://mvnrepository.com/artifact/org.hamcrest/hamcrest)
* [org.hamcrest:hamcrest-library](https://mvnrepository.com/artifact/org.hamcrest/hamcrest-library) ([HamcrestLibraryRule](src/main/java/org.gradlex/javaecosystem/capabilities/rules/HamcrestLibraryRule.java))
  * [org.hamcrest:hamcrest](https://mvnrepository.com/artifact/org.hamcrest/hamcrest)
* [postgresql:postgresql](https://mvnrepository.com/artifact/postgresql/postgresql) ([PostgresqlRule](src/main/java/org.gradlex/javaecosystem/capabilities/rules/PostgresqlRule.java))
  * [org.postgresql:postgresql](https://mvnrepository.com/artifact/org.postgresql/postgresql)
* [stax:stax-api](https://mvnrepository.com/artifact/stax/stax-api) ([StaxApiRule](src/main/java/org.gradlex/javaecosystem/capabilities/rules/StaxApiRule.java))
  * [org.codehaus.woodstox:stax2-api](https://mvnrepository.com/artifact/org.codehaus.woodstox/stax2-api)
  * [javax.xml.stream:stax-api](https://mvnrepository.com/artifact/javax.xml.stream/stax-api)
* [velocity:velocity](https://mvnrepository.com/artifact/velocity/velocity) ([VelocityRule](src/main/java/org.gradlex/javaecosystem/capabilities/rules/VelocityRule.java))
  * [org.apache.velocity:velocity](https://mvnrepository.com/artifact/org.apache.velocity/velocity)
  * [org.apache.velocity:velocity-engine-core](https://mvnrepository.com/artifact/org.apache.velocity/velocity-engine-core)
* [woodstox:wstx-asl](https://mvnrepository.com/artifact/woodstox/wstx-asl) ([WoodstoxAslRule](src/main/java/org.gradlex/javaecosystem/capabilities/rules/WoodstoxAslRule.java))
  * [org.codehaus.woodstox:wstx-asl](https://mvnrepository.com/artifact/org.codehaus.woodstox/wstx-asl)
  * [org.codehaus.woodstox:woodstox-core-asl](https://mvnrepository.com/artifact/org.codehaus.woodstox/woodstox-core-asl)
* [woodstox:wstx-lgpl](https://mvnrepository.com/artifact/woodstox/wstx-lgpl) ([WoodstoxLgplRule](src/main/java/org.gradlex/javaecosystem/capabilities/rules/WoodstoxLgplRule.java))
  * [org.codehaus.woodstox:wstx-lgpl](https://mvnrepository.com/artifact/org.codehaus.woodstox/wstx-lgpl)
  * [org.codehaus.woodstox:woodstox-core-lgpl](https://mvnrepository.com/artifact/org.codehaus.woodstox/woodstox-core-lgpl)

<!-- END_GENERATED -->

## Something seems to be missing

This plugin collects rules that universally apply in the Java ecosystem.
That means, that the information this plugin adds would ideally be already published in the metadata of the corresponding components.
The idea is that _every_ Java project can apply this plugin to avoid certain 'dependency hell' situations.
Even if the project does not use any of the components this plugin affects directly, transitive dependency might bring in components that cause conflicts.

At the moment this plugin is only covering a fraction of the components on Maven Central that miss capability information.
If you encounter more cases, please...

### ...contribute!

If you use this plugin and think it is missing a rule for a well-known component (or that a rule is incomplete/wrong), please let us know by

- [Opening an Issue](https://github.com/gradlex-org/java-ecosystem-capabilities/blob/main/issues/new)
- [Providing a PR](https://github.com/gradlex-org/java-ecosystem-capabilities/pulls) - for this you should look at the [existing rules](src/main/java/org.gradlex/javaecosystem/capabilities/rules) and follow the same patterns for new rules

Please make sure, you clearly state which **Capability** it is about and which **Components** provide the Capability.


### Special Case: Logging Libraries

This plugin does not contain rules for logging libraries, which is a specific area in which conflicts occur regularly.
There is [a separate plugin](https://github.com/ljacomet/logging-capabilities) covering this topic by adding capabilities to the components of well-known logging APIs and implementations.
Please apply that plugin in addition to this one:

```
plugins {
    ...
    id("org.gradlex.java-ecosystem-capabilities")
    id("dev.jacomet.logging-capabilities")
}
```

## I maintain a Component on Maven Central - How can I publish Capability information myself?

It would be great to see more components publishing capability information directly.
If you wonder how you could do it, here is how:

### Publishing with Gradle

Assuming the component you are publishing is `org.ow2.asm:asm`.
You add the `asm:asm` capability as follows:

```
configurations {
    apiElements {
        outgoing {
            capability("${project.group}:${project.name}:${project.verson}") // keep default capability 'org.ow2.asm:asm'
            capability("asm:asm:${project.verson}")                          // add 'asm:asm'
        }
    }
    runtimeElements {
        outgoing {
            capability("${project.group}:${project.name}:${project.verson}") // keep default capability 'org.ow2.asm:asm'
            capability("asm:asm:${project.verson}")                          // add 'asm:asm'
        }
    }
}
```

See also: [Documentation in Gradle Manual](https://docs.gradle.org/current/userguide/component_capabilities.html#sec:declaring-additional-capabilities-for-a-local-component)

### Publishing with Maven

Assuming the component you are publishing is `org.ow2.asm:asm`.
You add the `asm:asm` capability as follows:

```
<!-- do_not_remove: published-with-gradle-metadata -->

<build>
  <plugins>
    <plugin>
      <groupId>de.jjohannes</groupId>
      <artifactId>gradle-module-metadata-maven-plugin</artifactId>
      <version>0.2.0</version>
      <executions>
        <execution>
          <goals>
            <goal>gmm</goal>
          </goals>
        </execution>
      </executions>
      <configuration>
        <capabilities>
          <capability>
            <groupId>asm</groupId>
            <artifactId>asm</artifactId>
          </capability>
        </capabilities>
      </configuration>
    </plugin>
  </plugins>
</build>

```

See also: [Documentation of `gradle-module-metadata-maven-plugin` Maven Plugin](https://github.com/gradlex-org/gradle-module-metadata-maven-plugin#defining-capabilities)

# Disclaimer

Gradle and the Gradle logo are trademarks of Gradle, Inc.
The GradleX project is not endorsed by, affiliated with, or associated with Gradle or Gradle, Inc. in any way.
